typedef union {
    char *s;
    char c;
    shape_t shape;
    sentry_t sentry;
    int num;
} YYSTYPE;
#define	YSHAPES	258
#define	YELASTIC	259
#define	YPADDING	260
#define	YSAMPLE	261
#define	YENDSAMPLE	262
#define	YBOX	263
#define	YEND	264
#define	YUNREC	265
#define	YREPLACE	266
#define	YREVERSE	267
#define	YTO	268
#define	YWITH	269
#define	YCHGDEL	270
#define	KEYWORD	271
#define	WORD	272
#define	STRING	273
#define	SHAPE	274
#define	YNUMBER	275
#define	YRXPFLAG	276
#define	YDELWORD	277


extern YYSTYPE yylval;
