/*
 *  File:             remove.h
 *  Project Main:     boxes.c
 *  Date created:     June 23, 1999 (Wednesday, 21:00h)
 *  Author:           Copyright (C) 1999 Thomas Jensen
 *                    tsjensen@stud.informatik.uni-erlangen.de
 *  Version:          $Id: remove.h,v 1.2 1999/07/23 16:16:40 tsjensen Exp tsjensen $
 *  Language:         ANSI C
 *  Purpose:          Box removal, i.e. the deletion of boxes
 *
 *  Remarks: o This program is free software; you can redistribute it and/or
 *             modify it under the terms of the GNU General Public License as
 *             published by the Free Software Foundation; either version 2 of
 *             the License, or (at your option) any later version.
 *           o This program is distributed in the hope that it will be useful,
 *             but WITHOUT ANY WARRANTY; without even the implied warranty of
 *             MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *             GNU General Public License for more details.
 *           o You should have received a copy of the GNU General Public
 *             License along with this program; if not, write to the Free
 *             Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *             MA 02111-1307  USA
 *
 *  Revision History:
 *
 *    $Log: remove.h,v $
 *    Revision 1.2  1999/07/23 16:16:40  tsjensen
 *    Added GNU GPL disclaimer
 *
 *    Revision 1.1  1999/06/23 19:15:00  tsjensen
 *    Initial revision
 *
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#ifndef REMOVE_H
#define REMOVE_H


int remove_box();
void output_input();


#endif /*REMOVE_H*/

/*EOF*/                                          /* vim: set cindent sw=4: */
